"""
Clash 安装器
"""

import shutil
from pathlib import Path
from typing import Optional, Dict, Any
from rich.progress import Progress, SpinnerColumn, TextColumn, BarColumn, TaskProgressColumn

from ..constants import (
    CLASH_BASE_DIR, CLASH_BIN_DIR, CLASH_CONFIG_DIR, CLASH_LOG_DIR,
    BIN_MIHOMO, BIN_CLASH, BIN_YQ, BIN_SUBCONVERTER_DIR, BIN_SUBCONVERTER,
    DOWNLOAD_URLS, SUPPORTED_ARCHITECTURES
)
from ..exceptions import InstallationError, NetworkError
from ..utils import (
    check_root_permission, get_architecture, download_file, extract_archive,
    success_message, error_message, info_message, console
)


class ClashInstaller:
    """Clash 安装器"""
    
    def __init__(self):
        self.arch = get_architecture()
        if self.arch not in SUPPORTED_ARCHITECTURES:
            raise InstallationError(f"不支持的架构: {self.arch}")
    
    def is_installed(self) -> bool:
        """检查是否已安装"""
        return (BIN_MIHOMO.exists() or BIN_CLASH.exists()) and BIN_YQ.exists()
    
    def get_installation_info(self) -> Dict[str, Any]:
        """获取安装信息"""
        return {
            'installed': self.is_installed(),
            'base_dir': str(CLASH_BASE_DIR),
            'bin_dir': str(CLASH_BIN_DIR),
            'config_dir': str(CLASH_CONFIG_DIR),
            'log_dir': str(CLASH_LOG_DIR),
            'mihomo_exists': BIN_MIHOMO.exists(),
            'clash_exists': BIN_CLASH.exists(),
            'yq_exists': BIN_YQ.exists(),
            'subconverter_exists': BIN_SUBCONVERTER.exists(),
            'architecture': self.arch,
        }
    
    def install(self, kernel: str = 'mihomo', subscription_url: Optional[str] = None) -> None:
        """安装 Clash"""
        check_root_permission()
        
        if self.is_installed():
            info_message("Clash 已安装，如需重新安装请先卸载")
            return
        
        try:
            with Progress(
                SpinnerColumn(),
                TextColumn("[progress.description]{task.description}"),
                BarColumn(),
                TaskProgressColumn(),
                console=console
            ) as progress:
                
                # 创建目录
                task1 = progress.add_task("创建目录结构...", total=1)
                self._create_directories()
                progress.update(task1, completed=1)
                
                # 下载和安装内核
                task2 = progress.add_task(f"安装 {kernel} 内核...", total=1)
                self._install_kernel(kernel)
                progress.update(task2, completed=1)
                
                # 安装工具
                task3 = progress.add_task("安装辅助工具...", total=3)
                self._install_yq()
                progress.update(task3, advance=1)
                
                self._install_subconverter()
                progress.update(task3, advance=1)
                
                self._install_ui()
                progress.update(task3, advance=1)
                
                # 下载 GeoIP 数据库
                task4 = progress.add_task("下载 GeoIP 数据库...", total=1)
                self._install_geoip()
                progress.update(task4, completed=1)
                
                # 初始化配置
                task5 = progress.add_task("初始化配置...", total=1)
                self._initialize_config(subscription_url)
                progress.update(task5, completed=1)
            
            success_message("Clash 安装完成！")
            
        except Exception as e:
            error_message(f"安装失败: {e}")
            # 清理安装文件
            self._cleanup_installation()
            raise InstallationError(f"安装失败: {e}")
    
    def uninstall(self) -> None:
        """卸载 Clash"""
        check_root_permission()
        
        try:
            # 停止服务
            from .service import ClashService
            service = ClashService()
            if service.is_running():
                service.stop()
            if service.is_installed():
                service.uninstall_service()
            
            # 删除文件
            if CLASH_BASE_DIR.exists():
                shutil.rmtree(CLASH_BASE_DIR)
            
            # 清理 Shell 集成
            self._cleanup_shell_integration()
            
            success_message("Clash 卸载完成")
            
        except Exception as e:
            raise InstallationError(f"卸载失败: {e}")
    
    def _create_directories(self) -> None:
        """创建目录结构"""
        directories = [
            CLASH_BASE_DIR,
            CLASH_BIN_DIR,
            CLASH_CONFIG_DIR,
            CLASH_LOG_DIR,
            BIN_SUBCONVERTER_DIR,
        ]
        
        for directory in directories:
            directory.mkdir(parents=True, exist_ok=True)
    
    def _install_kernel(self, kernel: str) -> None:
        """安装内核"""
        if kernel not in ['mihomo', 'clash']:
            raise InstallationError(f"不支持的内核: {kernel}")
        
        if kernel not in DOWNLOAD_URLS:
            raise InstallationError(f"未找到 {kernel} 的下载链接")
        
        if self.arch not in DOWNLOAD_URLS[kernel]:
            raise InstallationError(f"{kernel} 不支持架构 {self.arch}")
        
        url = DOWNLOAD_URLS[kernel][self.arch]
        filename = Path(url).name
        download_path = CLASH_BIN_DIR / filename
        
        # 下载
        download_file(url, download_path, show_progress=False)
        
        # 解压
        extract_archive(download_path, CLASH_BIN_DIR)
        
        # 设置权限
        if kernel == 'mihomo':
            if BIN_MIHOMO.exists():
                BIN_MIHOMO.chmod(0o755)
            else:
                # 查找解压出的文件
                for file in CLASH_BIN_DIR.glob('mihomo*'):
                    if file.is_file() and not file.suffix:
                        file.rename(BIN_MIHOMO)
                        BIN_MIHOMO.chmod(0o755)
                        break
        else:
            if BIN_CLASH.exists():
                BIN_CLASH.chmod(0o755)
            else:
                for file in CLASH_BIN_DIR.glob('clash*'):
                    if file.is_file() and not file.suffix:
                        file.rename(BIN_CLASH)
                        BIN_CLASH.chmod(0o755)
                        break
        
        # 清理下载文件
        download_path.unlink(missing_ok=True)
    
    def _install_yq(self) -> None:
        """安装 yq"""
        url = DOWNLOAD_URLS['yq']
        filename = Path(url).name
        download_path = CLASH_BIN_DIR / filename
        
        # 下载
        download_file(url, download_path, show_progress=False)
        
        # 解压
        extract_archive(download_path, CLASH_BIN_DIR)
        
        # 查找并重命名 yq 文件
        for file in CLASH_BIN_DIR.glob('yq_*'):
            if file.is_file():
                file.rename(BIN_YQ)
                BIN_YQ.chmod(0o755)
                break
        
        # 清理下载文件
        download_path.unlink(missing_ok=True)
    
    def _install_subconverter(self) -> None:
        """安装 subconverter"""
        url = DOWNLOAD_URLS['subconverter']
        filename = Path(url).name
        download_path = CLASH_BIN_DIR / filename
        
        # 下载
        download_file(url, download_path, show_progress=False)
        
        # 解压到 subconverter 目录
        extract_archive(download_path, BIN_SUBCONVERTER_DIR)
        
        # 设置权限
        if BIN_SUBCONVERTER.exists():
            BIN_SUBCONVERTER.chmod(0o755)
        
        # 清理下载文件
        download_path.unlink(missing_ok=True)
    
    def _install_ui(self) -> None:
        """安装 Web UI"""
        url = DOWNLOAD_URLS['yacd']
        filename = Path(url).name
        download_path = CLASH_BASE_DIR / filename
        
        # 下载
        download_file(url, download_path, show_progress=False)
        
        # 解压到 public 目录
        ui_dir = CLASH_BASE_DIR / 'public'
        extract_archive(download_path, ui_dir)
        
        # 清理下载文件
        download_path.unlink(missing_ok=True)
    
    def _install_geoip(self) -> None:
        """安装 GeoIP 数据库"""
        url = DOWNLOAD_URLS['country_mmdb']
        download_path = CLASH_BASE_DIR / 'Country.mmdb'
        
        # 下载
        download_file(url, download_path, show_progress=False)
    
    def _initialize_config(self, subscription_url: Optional[str] = None) -> None:
        """初始化配置"""
        from .config import ClashConfig
        from .service import ClashService
        from .proxy import ProxyManager
        
        # 初始化配置管理器
        config_manager = ClashConfig()
        config_manager.init_mixin_config()
        
        # 如果提供了订阅链接，下载配置
        if subscription_url:
            try:
                config_manager.download_config(subscription_url)
                config_manager.merge_configs()
            except Exception as e:
                error_message(f"下载订阅配置失败: {e}")
                info_message("请稍后使用 'clash-cli update' 命令更新订阅")
        
        # 安装服务
        service = ClashService()
        service.install_service()
        service.enable()
        
        # 设置 Shell 集成
        proxy_manager = ProxyManager()
        proxy_manager.setup_shell_integration()
    
    def _cleanup_installation(self) -> None:
        """清理安装文件"""
        try:
            if CLASH_BASE_DIR.exists():
                shutil.rmtree(CLASH_BASE_DIR)
        except Exception:
            pass
    
    def _cleanup_shell_integration(self) -> None:
        """清理 Shell 集成"""
        try:
            shell_files = [
                Path.home() / '.bashrc',
                Path.home() / '.zshrc',
                Path.home() / '.config/fish/conf.d/clash-cli.fish',
            ]
            
            for shell_file in shell_files:
                if shell_file.exists():
                    # 读取文件内容
                    with open(shell_file, 'r') as f:
                        lines = f.readlines()
                    
                    # 过滤掉 clash-cli 相关行
                    filtered_lines = []
                    skip_section = False
                    
                    for line in lines:
                        if '# clash-cli integration' in line:
                            skip_section = True
                            continue
                        elif skip_section and line.strip() == '':
                            skip_section = False
                            continue
                        elif not skip_section:
                            filtered_lines.append(line)
                    
                    # 写回文件
                    with open(shell_file, 'w') as f:
                        f.writelines(filtered_lines)
                        
        except Exception:
            pass
